from setuptools import setup, find_packages

setup(
    name='tengi',
    version='dev',
    url='',
    author='dev',
    author_email='',
    description='',
    packages=find_packages(),    
    install_requires=[],
)
