
# Taken from: https://core.telegram.org/api/errors#400-bad-request
#   some (like CONFLICT) are googled
BAD_REQUEST = 400
UNAUTHORIZED = 401
FORBIDDEN = 403  # User blocked the bot
NOT_FOUND = 404
NOT_ACCEPTABLE = 406
CONFLICT = 409
FLOOD = 420
TOO_MANY_REQUESTS = 429
INTERNAL = 500
BAD_GATEWAY = 502  # Telegram is temporary unavailable
